/*  $Id: prom_defs.h,v 1.21 2006/04/27 20:49:36 adams Exp $ */
/*  Author:             Mark F. Adams                       */
/* Copyright (c) 2000 by Mark F. Adams                      */
/*  Filename:           prom_defs.hh                        */
/*                                                          */
#ifndef __PROM_DEFS_H__
#define __PROM_DEFS_H__

// add PETSc if Petra is not used
#if !defined(PROM_USE_PETRA) 
#if !defined(PROM_HAVE_PETSC)
#define PROM_HAVE_PETSC
#endif
#if !defined(PROM_USE_PETSC)
#define PROM_USE_PETSC
#endif
#endif

//#define assert(ignore) ((void)0)
//#define NDEBUG
#include <assert.h>
#include <string.h>

//#include <stdio.h>
#include <stdlib.h>

#if defined(HPM) || defined(HPM_MM) || defined(HPM_F)
#include "libhpm.H"
#endif /* HPM */

#if defined(PROM_HAVE_PETSC)

#include "petsc.h"
#include "petscvec.h" // needed for athena
#include "petscmat.h" // needed for kkt solver

#else // no PETSc at all -- PETSc infrestructure replacements

#define INSERT_VALUES 0
#define ADD_VALUES 1

#define PetscMalloc(t,p) ((p=malloc(t)) == NULL)
int PetscFree( void *p ){ free(p); return 0; }
#define PetscGetResidentSetSize(t) (int)(*t = 0.0)
#define PetscFunctionReturn(a) return (a)
#define PetscFunctionReturnVoid() return
typedef bool PetscTruth;
typedef double PetscLogDouble;
int PetscPrintf(MPI_Comm comm,const char format[],...);
int PetscMemzero(void *a, int n){
  if (n < 0) SETERRQ(1,"Memory length must be >= 0");
  if (n > 0) memset(a,0,n);
  return 0;
}

#define CHKERRQ(a) if(a!=0){printf("\t\tERROR: in file %s line %d\n",__FILE__,__LINE__); return a;} 
#define SETERRQ(a,s){printf("\t\tERROR: %s in file %s line %d\n",s,__FILE__,__LINE__);return a;}
#define SETERRQ1(a,s,d){printf("\t\tERROR: %s in file %s line %d, %d\n",s,__FILE__,__LINE__,d);return a;}
#define SETERRQ2(a,s,d,d2){printf("\t\tERROR: %s in file %s line %d, %d %d\n",s,__FILE__,__LINE__,d,d2);return a;}

#endif

#ifndef TRUE
#define TRUE true
#endif

#ifndef FALSE
#define FALSE false
#endif

// METIS
#if defined(PROM_HAVE_METIS)
#ifndef __METIS_H__
#define __METIS_H__
#if defined(__cplusplus)
extern "C"{
#endif
  //#include "metis.h"
#include "parmetis.h"
#if defined(__cplusplus)
}
#endif 
#endif // __METIS_H__
#else
typedef int idxtype;
#endif // PROM_HAVE_METIS

// Vampir
#if defined(PROM_USE_VAMPIRE)
#define PROM_HAVE_VAMPIRE
#if defined(__cplusplus)
extern "C"{
#endif
#include "VT.h"  
#if defined(__cplusplus)
}
#endif
#endif

typedef enum NdStTAG{PROM_NOTUSED=0,PROM_DELETED,PROM_UNKNOWN,PROM_SELECTED,PROM_NUMSTATE}PromNodeState;
typedef enum EltypeTAG{PROM_SOLID=0,PROM_PLANE,PROM_N_POLY,PROM_NOELEM,PROM_NUMELEMTYPES}PromElemType;
typedef enum NdtpopTAG{PROM_CORNER=0,PROM_EDGE,PROM_SURFACE,PROM_INTERIOR,PROM_NUMTOPO}PromNodeTopo;
typedef enum PCSideTAG{PROM_LEFT=0,PROM_RIGHT,PROM_SYMM,PROM_FLEX}PromPCSide;

typedef double gfloat;
typedef double shpfloat;

//#define PROM_DELETE_DAP
#define PROM_GHOST_REGION 11
#define PROM_NENMAX 27
#define PROM_MAXPAR_HARD 1024
#define PROM_MAXPAR_SOFT 100
#define PROM_MAXNIEGH 2048
#define PROM_NUMENV 8
#define PROM_USE_NODAL_SMAGG
#define PROM_DIAG_FACTOR 1.e-12
#define PROM_RAP_SHIFT 1.e-10
#define PROM_BIG_NUM 1.e308
//#define PROM_TRACE_PC_SETUP

/* MPI buffers */
#if defined(PARCH_t3d)
/* typedef double sendfloat; */
#define PROM_BSZ 8192
#else
/* typedef double sendfloat; */
#define PROM_BSZ 8192
#endif

enum{LOG_FEI_INIT_STR,LOG_FEI_PROM_SETUP,LOG_FEI_MATRIX_SETUP,LOG_FEI_CR_BC,LOG_PARTITION,LOG_RESTRICTION,LOG_FEI_SOLVE,LOG_FEI_ASSEM,LOG_METIS,LOG_FEI_MAT_AXPY,LOG_FEI_RESETCR,LOG_SYMB_NUMR,LOG_NEW_GRID,LOG_FINALIZE,LOG_MAT_TRI_PROD, LOG_SLES_SETUP,LOG_PROM,LOG_FEAP,LOG_ASSEMBLE,LOG_COARSEN,LOG_SYMB_INIT_FEAP,LOG_NEW_COARSE,LOG_TRIANGULATE,LOG_FIXMESH,LOG_EXACTADJ,LOG_MAPS,LOG_MIS0,LOG_MIS1,LOG_MIS2,LOG_MIS3,LOG_MIS4,LOG_MIS5,LOG_MIS6,LOG_MIS7,LOG_REPART,LOG_AUX1,LOG_AUX2,LOG_AUX3,LOG_TOT};

typedef enum{PROM_READ,PROM_WRITE,PROM_PRINT}PromArchiveType;

#define CHKNEW(v,sz){if(!v){PetscPrintf(MPI_COMM_SELF,"[?]ERROR: %s, line %d, file %s failed to alloc %d bytes\n", __FUNCT__,__LINE__,__FILE__,sz);SETERRQ(1,"new memory error");}}
#define CHKNEW_void(v,sz){if(!v){PetscPrintf(MPI_COMM_SELF,"[?]ERROR %s, line %d, file %s: failed to alloc %d bytes\n", __FUNCT__, __LINE__, __FILE__, sz);}}

// FORTRAN 
#ifdef PETSC_HAVE_FORTRAN_CAPS
#define writer_       WRITER
#define feap_         FEAP
#define formfe_       FORMFE
#define pmacr1_       PMACR1
#define pmacr2_       PMACR2
#define pmacr3_       PMACR3
#define feap_set_dt_  FEAP_SET_DT
#define feap_set_pcont_l_ FEAP_SET_PCONT_L
#define feap_pre_solve_  FEAP_PRE_SOLVE
#define feap_update_ FEAP_UPDATE  
#define feap_plastic_data_ FEAP_PLASTIC_DATA
#define feap_finalize_  FEAP_FINALIZE
#define pfeap_assemble_ PFEAP_ASSEMBLE
#define mgformfe_       MGFORMFE
#define dgetrf_       SGETRF
#define dgetrs_       SGETRS
#define dgeqrf_       SGEQRF
#define dorgqr_       SORGQR
#define olympus_assemble_ OLYMPUS_ASSEMBLE
#define write_disp_       WRITE_DISP
#define write_all_disp_   WRITE_ALL_DISP
#define update_geometry_  UPDATE_GEOMETRY
#define dgesvd_           SGESVD
#define feap_get_mesh_sizes_     FEAP_GET_MESH_SIZES
#define feap_get_mesh_data_      FEAP_GET_MESH_DATA
#define feap_get_node_map_      FEAP_GET_NODE_MAP
#define feap_get_nen_           FEAP_GET_NEN
#define feap_get_nnodes_        FEAP_GET_NNODES
#define feap_get_nelems_        FEAP_GET_NELEMS
#define feap_get_nmat_        FEAP_GET_NMAT
#define feap_get_elem_natributes_ FEAP_GET_ELEM_NATRIBUTES
#define feap_get_elem_atributes_ FEAP_GET_ELEM_ATRIBUTES 
#define feap_set_array_ FEAP_SET_ARRAY
#define feap_get_array_ FEAP_GET_ARRAY 
#define feap_get_neq_ FEAP_GET_NEQ 
#define feap_apply_q_ FEAP_APPLY_Q
#define feap_srot_ FEAP_SROT
#define dgesvf_ DGESVF
#elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
#define writer_       writer
#define feap_         feap
#define formfe_       formfe
#define pmacr1_       pmacr1
#define pmacr2_       pmacr2
#define pmacr3_       pmacr3
#define feap_set_dt_  feap_set_dt
#define feap_set_pcont_l_ feap_set_pcont_l
#define feap_pre_solve_  feap_pre_solve 
#define feap_update_ feap_update  
#define feap_plastic_data_ feap_plastic_data
#define feap_finalize_     feap_finalize
#define pfeap_assemble_    pfeap_assemble
#define mgformfe_ mgformfe
#define dgetrf_       dgetrf
#define dgetrs_       dgetrs
#define dgeqrf_       dgeqrf
#define dorgqr_       dorgqr
#define olympus_assemble_ olympus_assemble
#define write_disp_       write_disp
#define write_all_disp_   write_all_disp
#define update_geometry_  update_geometry
#define dgesvd_           dgesvd
#define feap_get_mesh_sizes_     feap_get_mesh_sizes
#define feap_get_mesh_data_      feap_get_mesh_data
#define feap_get_node_map_      feap_get_node_map
#define feap_get_nen_           feap_get_nen
#define feap_get_nnodes_        feap_get_nnodes
#define feap_get_nelems_        feap_get_nelems
#define feap_get_nmat_        feap_get_nmat
#define feap_get_elem_natributes_ feap_get_elem_natributes
#define feap_get_elem_atributes_ feap_get_elem_atributes
#define feap_set_array_ feap_set_array
#define feap_get_array_ feap_get_array
#define feap_get_neq_   feap_get_neq
#define feap_apply_q_   feap_apply_q
#define feap_srot_ feap_srot
#define dgesvf_ dgesvf
#elif defined(PETSC_HAVE_FORTRAN_UNDERSCORE_UNDERSCORE)
#define writer_       writer_
#define feap_         feap_
#define formfe_       formfe_
#define pmacr1_       pmacr1_
#define pmacr2_       pmacr2_
#define pmacr3_       pmacr3_
#define feap_set_dt_  feap_set_dt__
#define feap_set_pcont_l_ feap_set_pcont_l__
#define feap_pre_solve_  feap_pre_solve__
#define feap_update_ feap_update__
#define feap_plastic_data_ feap_plastic_data__
#define feap_finalize_ feap_finalize__
#define pfeap_assemble_ pfeap_assemble__
#define mgformfe_ mgformfe_
#define dgetrf_       dgetrf_
#define dgetrs_       dgetrs_
#define dgeqrf_       dgeqrf_
#define dorgqr_       dorgqr_
#define olympus_assemble_ olympus_assemble__
#define write_disp_       write_disp__
#define write_all_disp_   write_all_disp__
#define update_geometry_  update_geometry__
#define dgemm_            dgemm__
#define dgemv_            dgemv__
#define dgesvd_           dgesvd__
#define feap_get_mesh_sizes_     feap_get_mesh_sizes__
#define feap_get_mesh_data_      feap_get_mesh_data__
#define feap_get_node_map_      feap_get_node_map__
#define feap_get_nen_           feap_get_nen__
#define feap_get_nmat_        feap_get_nmat__
#define feap_get_elem_natributes_ feap_get_elem_natributes__
#define feap_get_elem_atributes_ feap_get_elem_atributes__
#define feap_get_nnodes_        feap_get_nnodes__
#define feap_get_nelems_        feap_get_nelems__
#define feap_set_array_ feap_set_array__
#define feap_get_array_ feap_get_array__
#define feap_get_neq_   feap_get_neq__
#define feap_apply_q_   feap_apply_q__
#define feap_srot_ feap_srot__
#define dgesvf_ dgesvf__
#endif

#endif // __PROM_DEFS_H__
